# HG changeset patch
# User Elliot Thomas <elliot.thomas@bbc.co.uk>
# Date 1543836716 0
#      Mon Dec 03 11:31:56 2018 +0000
# Node ID 28084fdcef2873df221b13d1a70ab04242e4edc5
# Parent  2117637f64e981e0e14c3a4b0509252fefd8a78a
Apply cache locking behaviour to stale cache entries.

Previously, this was only done for new cache entries.

diff -r 2117637f64e9 -r 28084fdcef28 src/http/ngx_http_cache.h
--- a/src/http/ngx_http_cache.h	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_cache.h	Mon Dec 03 11:31:56 2018 +0000
@@ -188,6 +188,7 @@
 ngx_int_t ngx_http_file_cache_create(ngx_http_request_t *r);
 void ngx_http_file_cache_create_key(ngx_http_request_t *r);
 ngx_int_t ngx_http_file_cache_open(ngx_http_request_t *r);
+ngx_int_t ngx_http_file_cache_lock(ngx_http_request_t *r, ngx_int_t rc);
 ngx_int_t ngx_http_file_cache_set_header(ngx_http_request_t *r, u_char *buf);
 void ngx_http_file_cache_update(ngx_http_request_t *r, ngx_temp_file_t *tf);
 void ngx_http_file_cache_update_header(ngx_http_request_t *r);
diff -r 2117637f64e9 -r 28084fdcef28 src/http/ngx_http_file_cache.c
--- a/src/http/ngx_http_file_cache.c	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_file_cache.c	Mon Dec 03 11:31:56 2018 +0000
@@ -11,8 +11,6 @@
 #include <ngx_md5.h>
 
 
-static ngx_int_t ngx_http_file_cache_lock(ngx_http_request_t *r,
-    ngx_http_cache_t *c);
 static void ngx_http_file_cache_lock_wait_handler(ngx_event_t *ev);
 static void ngx_http_file_cache_lock_wait(ngx_http_request_t *r,
     ngx_http_cache_t *c);
@@ -391,22 +389,21 @@
 
 done:
 
-    if (rv == NGX_DECLINED) {
-        return ngx_http_file_cache_lock(r, c);
-    }
-
     return rv;
 }
 
 
-static ngx_int_t
-ngx_http_file_cache_lock(ngx_http_request_t *r, ngx_http_cache_t *c)
+ngx_int_t
+ngx_http_file_cache_lock(ngx_http_request_t *r, ngx_int_t rc)
 {
     ngx_msec_t                 now, timer;
+    ngx_http_cache_t          *c;
     ngx_http_file_cache_t     *cache;
 
+    c = r->cache;
+
     if (!c->lock) {
-        return NGX_DECLINED;
+        return rc;
     }
 
     now = ngx_current_msec;
@@ -431,7 +428,7 @@
                    c->updating, c->wait_time);
 
     if (c->updating) {
-        return NGX_DECLINED;
+        return rc;
     }
 
     if (c->lock_timeout == 0) {
diff -r 2117637f64e9 -r 28084fdcef28 src/http/ngx_http_upstream.c
--- a/src/http/ngx_http_upstream.c	Tue Nov 27 17:40:21 2018 +0300
+++ b/src/http/ngx_http_upstream.c	Mon Dec 03 11:31:56 2018 +0000
@@ -923,6 +923,11 @@
         u->cache_status = NGX_HTTP_CACHE_HIT;
     }
 
+    if (rc == NGX_DECLINED || rc == NGX_HTTP_CACHE_STALE)
+    {
+        rc = ngx_http_file_cache_lock(r, rc);
+    }
+
     switch (rc) {
 
     case NGX_OK:
